function Distribution = TV(TimeTrace,Kernel,L,RegularizationParameter,options)
%--------------------------------------------------------------------------
% Total Variation Regularization. The functional employs the lower bound
% smooth approximation to ensure continuous differentiability of the
% function. 
%
%--------------------------------------------------------------------------
% L. Fabregas, 2018
%--------------------------------------------------------------------------
warning('on','all')
if strcmp(options.Solver,'cvx')
  if exist('cvx','file')~=2
    warning('CVX not found. Make sure it is installed properly.')
    options.Solver = 'fmincon';
  end
end
warning('off','all')

%--------------------------------------------------------------------------
% Preparation
%--------------------------------------------------------------------------
SolverOptions = optimset('GradObj','off','MaxFunEvals',200000,'Display','off','MaxIter',200000);
%Non-negative constraint
NonNegativityConstraint = zeros(1,length(TimeTrace));
TimeTrace = TimeTrace';
if strcmp(options.Solver,'fmincon')
  Distribution = ones(length(TimeTrace),1);
end

%--------------------------------------------------------------------------
% Regularization
%--------------------------------------------------------------------------
switch options.Solver
  case 'fmincon'
    %Run constricted functional minimization
    Distribution =  fmincon(@(Distribution) TVFunctional(TimeTrace,Distribution,L,Kernel,RegularizationParameter),Distribution,[],[],[],[],NonNegativityConstraint,[],[],SolverOptions);
  case 'cvx'
    SizeDistribution = length(TimeTrace);
    cvx_begin quiet
        variable Distribution(SizeDistribution)
        clear VectorialNorm
        Auxiliary = L*Distribution;
        % Vectorial norm has not implemented until MATLAB R2017b, do manually
        for i=1:length(Auxiliary)
            %Ignore preallocation warning to ensure 'cvx convex expression' class
            VectorialNorm(i) =  norm([Auxiliary(i) 1e-12]);
        end
        minimize( 1/2*square_pos(norm(Kernel*Distribution - TimeTrace,'fro')) + RegularizationParameter^2*sum(VectorialNorm) )
        subject to
            Distribution >= 0;
    cvx_end
end

end

function [Functional] = TVFunctional(TimeTrace,Distribution,L,Kernel,RegularizationParameter)
Functional = 1/2*norm(TimeTrace - Kernel*Distribution)^2 + RegularizationParameter^2*sum(sqrt((L*Distribution).^2 + 1e-24));
end